from posixpath import expanduser
import sys
import argparse
from gettext import gettext as _
from os.path import isfile
from gi.repository import Gtk, Gdk, Gio, GLib
from gfeeds.confManager import ConfManager
from gfeeds.feeds_manager import FeedsManager
from gfeeds.app_window import GFeedsAppWindow
from gfeeds.preferences_window import show_preferences_window
from gfeeds.opml_manager import (
    feeds_list_to_opml,
    add_feeds_from_opml,
    opml_to_rss_list
)
from gfeeds.opml_file_chooser import (
    GFeedsOpmlFileChooserDialog,
    GFeedsOpmlSavePathChooserDialog
)
from gfeeds.manage_feeds_window import GFeedsManageFeedsWindow
from gfeeds.scrolled_dialog import ScrolledDialog
from gfeeds.shortcuts_window import show_shortcuts_window
from gfeeds.util.rss_link_from_file import get_feed_link_from_file
from gfeeds.base_app import BaseApp, AppAction
from xml.sax.saxutils import escape


class GFeedsApplication(BaseApp):
    def __init__(self):
        self.confman = ConfManager()
        super().__init__(
            app_id='org.gabmus.gfeeds',
            app_name='Feeds',
            app_actions=[
                AppAction(
                    name='show_read_items',
                    func=self.show_read_items,
                    accel='<Control>h',
                    stateful=True,
                    state_type=AppAction.StateType.BOOL,
                    state_default=self.confman.conf['show_read_items']
                ),
                AppAction(
                    name='show_empty_feeds',
                    func=self.show_empty_feeds,
                    stateful=True,
                    state_type=AppAction.StateType.BOOL,
                    state_default=self.confman.conf['show_empty_feeds']
                ),
                AppAction(
                    name='view_mode_change',
                    func=self.view_mode_change,
                    stateful=True,
                    state_type=AppAction.StateType.RADIO,
                    state_default=self.confman.conf['default_view']
                ),
                AppAction(
                    name='set_all_read',
                    func=self.set_all_read,
                    accel='<Control>m'
                ),
                AppAction(
                    name='set_all_unread',
                    func=self.set_all_unread
                ),
                AppAction(
                    name='manage_feeds',
                    func=self.manage_feeds
                ),
                AppAction(
                    name='import_opml',
                    func=self.import_opml
                ),
                AppAction(
                    name='export_opml',
                    func=self.export_opml
                ),
                AppAction(
                    name='settings',
                    func=lambda *args: show_preferences_window(self.window),
                    accel='<Primary>comma'
                ),
                AppAction(
                    name='shortcuts',
                    func=lambda *args: show_shortcuts_window(self.window),
                    accel='<Primary>question'
                ),
                AppAction(
                    name='about',
                    func=self.show_about_dialog
                ),
                AppAction(
                    name='quit',
                    func=self.on_destroy_window,
                    accel='<Primary>q'
                ),
                AppAction(
                    name='open_externally',
                    func=self.open_externally,
                    accel='<Control>o'
                ),
                AppAction(
                    name='open_media_player',
                    func=self.open_media_player
                ),
                AppAction(
                    name='copy_link',
                    func=self.copy_link
                )
            ],
            flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE,
            css_resource='/org/gabmus/gfeeds/ui/gtk_style.css'
        )
        self.feedman = FeedsManager()

    def do_startup(self):
        super().do_startup()
        self.window = GFeedsAppWindow(self)
        self.window.connect('close-request', self.on_destroy_window)
        self.add_window(self.window)
        self.feedman.refresh(
            get_cached=not self.confman.conf['refresh_on_startup'],
            is_startup=True
        )

    def open_media_player(self, *args):
        self.window.leaflet.webview.action_open_media_player()

    def open_externally(self, *args):
        self.window.leaflet.webview.open_externally()

    def copy_link(self, *args):
        Gdk.Display.get_default().get_clipboard().set(
            self.window.leaflet.webview.uri
        )
        self.window.leaflet.webview.show_notif()

    def view_mode_change(
            self, action: Gio.SimpleAction, target: GLib.Variant, *args
    ):
        action.change_state(target)
        target_s = str(target).strip("'")
        if target_s not in ['webview', 'reader', 'rsscont']:
            target_s = 'webview'
        self.window.leaflet.on_view_mode_change(target_s)
        self.confman.conf['default_view'] = target_s
        self.confman.save_conf()

    def show_read_items(self, action: Gio.SimpleAction, *args):
        action.change_state(
            GLib.Variant.new_boolean(not action.get_state().get_boolean())
        )
        self.confman.conf['show_read_items'] = action.get_state().get_boolean()
        self.confman.emit('gfeeds_show_read_changed', '')

    def show_empty_feeds(self, action: Gio.SimpleAction, *args):
        action.change_state(
            GLib.Variant.new_boolean(not action.get_state().get_boolean())
        )
        self.confman.conf['show_empty_feeds'] = \
            action.get_state().get_boolean()
        self.confman.emit('gfeeds_show_empty_feeds_changed', '')

    def set_all_read(self, *args):
        self.window.leaflet.sidebar.listview_sw.set_all_read_state(True)

    def set_all_unread(self, *args):
        self.window.leaflet.sidebar.listview_sw.set_all_read_state(False)

    def manage_feeds(self, *args):
        mf_win = GFeedsManageFeedsWindow(
            self.window
        )
        mf_win.present()

    def import_opml(self, *args):
        dialog = GFeedsOpmlFileChooserDialog(self.window)

        def on_response(_dialog, res):
            if res == Gtk.ResponseType.ACCEPT:
                add_feeds_from_opml(dialog.get_file().get_path())

        dialog.connect('response', on_response)
        dialog.show()

    def export_opml(self, *args):
        dialog = GFeedsOpmlSavePathChooserDialog(self.window)

        def on_response(_dialog, res):
            if res == Gtk.ResponseType.ACCEPT:
                save_path = dialog.get_file().get_path()
                if not save_path.lower().endswith('.opml'):
                    save_path += '.opml'
                opml_out = feeds_list_to_opml(
                    self.feedman.feed_store.sort_store
                )
                with open(save_path, 'w') as fd:
                    fd.write(opml_out)

        dialog.connect('response', on_response)
        dialog.show()

    def show_about_dialog(self, *args):
        about_builder = Gtk.Builder.new_from_resource(
            '/org/gabmus/gfeeds/aboutdialog.ui'
        )
        dialog = about_builder.get_object('aboutdialog')
        dialog.set_modal(True)
        dialog.set_transient_for(self.window)
        dialog.present()

    def on_destroy_window(self, *args):
        self.window.on_destroy()
        self.quit()

    def do_activate(self):
        super().do_activate()
        self.window.present()
        # self.feedman.refresh(get_cached=True)
        if self.args:
            if self.args.argurl:
                abspath = self.args.argurl.strip()
                if abspath.lower().startswith('file:///'):
                    abspath = self.args.argurl.removeprefix('file://')
                if isfile(expanduser(abspath)):
                    if abspath.lower().endswith('.opml'):
                        dialog = ScrolledDialog(
                            transient_for=self.window,
                            title=_('Do you want to import these feeds?'),
                            message=escape('\n'.join([
                                f['feed']
                                for f in opml_to_rss_list(abspath)
                            ]))
                        )

                        def on_response(_dialog, res):
                            _dialog.close()
                            if res == Gtk.ResponseType.YES:
                                add_feeds_from_opml(abspath)

                        dialog.connect('response', on_response)
                        dialog.present()
                    else:
                        # why no check for extension here?
                        # some websites have feeds without extension
                        # dumb but that's what it is
                        self.args.argurl = get_feed_link_from_file(
                            abspath
                        ) or ''
                if (
                        self.args.argurl.lower().startswith('http://') or
                        self.args.argurl.lower().startswith('https://')
                ):
                    dialog = ScrolledDialog(
                        transient_for=self.window,
                        title=_('Do you want to import this feed?'),
                        message=escape(self.args.argurl),
                    )
                    argurl = self.args.argurl

                    def on_response(_dialog, res):
                        _dialog.close()
                        if res == Gtk.ResponseType.YES:
                            self.feedman.add_feed(argurl)

                    dialog.connect('response', on_response)
                    dialog.present()
                else:
                    print('This file is not supported')
            self.args = None

    def do_command_line(self, args: Gio.ApplicationCommandLine):
        """
        GTK.Application command line handler
        called if Gio.ApplicationFlags.HANDLES_COMMAND_LINE is set.
        must call the self.do_activate() to get the application up and running.
        """
        # call the default commandline handler
        # not required anymore?
        # Gtk.Application.do_command_line(self, args)
        # make a command line parser
        parser = argparse.ArgumentParser()
        parser.add_argument(
            'argurl',
            metavar=_('url'),
            type=str,
            nargs='?',
            help=_('opml file local url or rss remote url to import')
        )
        # parse the command line stored in args,
        # but skip the first element (the filename)
        self.args = parser.parse_args(args.get_arguments()[1:])
        # call the main program do_activate() to start up the app
        self.do_activate()
        return 0


def main():

    application = GFeedsApplication()

    try:
        ret = application.run(sys.argv)
    except SystemExit as e:
        ret = e.code

    sys.exit(ret)


if __name__ == '__main__':
    main()
